package com.fuzamei.service;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.fuzamei.constant.OperationTypeId;
import com.fuzamei.constant.Role;
import com.fuzamei.dao.PromptLetterDao;
import com.fuzamei.entity.PromptLetter;
import com.fuzamei.entity.UserDetail;
import com.fuzamei.utils.PageDTO;

@Service
public class PromptLetterService {
	@Autowired
	private PromptLetterDao promptLetterDao;

	/**
	 * 提示函 管理人接收列表
	 * 
	 * @param map
	 * @return
	 */
	public PageDTO selectPromptLetter(Map<String, Object> map) {// 查询风险提示函
		PageDTO pageDto = new PageDTO(); // 创建分页对象
		UserDetail userDetail = (UserDetail) map.get("userDetail");
		if(userDetail.getRole_name().contains(Role.ADMIN)){
			//将管理员的id号封装进map到数据库查询
			map.put("receiverId", userDetail.getUser_id());
			List<PromptLetter> list = promptLetterDao.queryPromptLetterByAdmin(map);
			int count = promptLetterDao.findAllInformation(map);
			pageDto.setTotal(count);
			pageDto.setRows(list);
			return pageDto;
		}else{
			//除了管理员之外所有人都能查看，以风险岗的视角去查看
			List<PromptLetter> list = promptLetterDao.queryPromptLetterByVenture(map);
			int count = promptLetterDao.findAllInformation(map);
			pageDto.setTotal(count);
			pageDto.setRows(list);
			return pageDto;
		}
		/*if(userDetail.getRole_name().contains(Role.VENTURE_MANAGEMENT_POS)){
			List<PromptLetter> list = promptLetterDao.queryPromptLetterByVenture(map);
			int count = promptLetterDao.findAllInformation(map);
			pageDto.setTotal(count);
			pageDto.setRows(list);
			return pageDto;
		}
		throw new RuntimeException("无权限操作");*/
	}

	/**
	 * 提示函 发函 上传文件
	 * 
	 * @param map
	 */
	@Transactional(rollbackFor = Exception.class)
	public void insertTiShiHanOrUserOrOtherTable(Map<String, Object> map) {
		long currentTime = System.currentTimeMillis();
		UserDetail userDetailOfVenture = (UserDetail) map.get("userDetailOfVenture");
		UserDetail userDetailOfAdmin = (UserDetail) map.get("userDetailOfAdmin");
		int attachmentId = (Integer) map.get("attachmentId");
		String attachmentName = (String) map.get("attachmentName");
		String hash = (String) map.get("hash");
		String url = (String) map.get("url");
		Map<String, Object> mapToDao4PromptLetter = new LinkedHashMap<String, Object>();
		mapToDao4PromptLetter.put("prompt_letter_id", attachmentId);// 提示函 id
		mapToDao4PromptLetter.put("prompt_letter_name", attachmentName);// 提示函名称 目前写的是文件名称
		mapToDao4PromptLetter.put("send_person", userDetailOfVenture.getUser_id());// 发送人暂时写死的 ?? 是不是应该获取登陆人的id？？？？
		mapToDao4PromptLetter.put("receive_person", userDetailOfAdmin.getUser_id());// 接收人应该是？？ ？？？ 接收人写哪个
		mapToDao4PromptLetter.put("send_time", currentTime);// 发送时间
		mapToDao4PromptLetter.put("receive_time", currentTime);// 接收时间
		mapToDao4PromptLetter.put("hash", hash);// hash值可以插入了
		promptLetterDao.insertIntoPromptLetterTable(mapToDao4PromptLetter);// 将账号插入提示函表

		Map<String, Object> mapToDao4Attachment = new LinkedHashMap<String, Object>();
		mapToDao4Attachment.put("attachmentId", attachmentId);// 附件id 目前写的是上面输入的管理人账号一样
		mapToDao4Attachment.put("attachmentName", attachmentName);// 附件名称 拿到了
		mapToDao4Attachment.put("url", url);// 附件url 后面有下载的 需要地址
		mapToDao4Attachment.put("upload_person_id", userDetailOfVenture.getUser_id());// 附件人上传的id这里写的是死值 待改动
		mapToDao4Attachment.put("create_time", currentTime);// 附件添加时间，这里应该写发送时间
		promptLetterDao.insertIntoAttachmentTable(mapToDao4Attachment);// 将数据添加到附件表数据库里

		Map<String, Object> mapToOperation = new LinkedHashMap<String, Object>();
		mapToOperation.put("operatorTypeId", OperationTypeId.UPLOAD_PROMPT_LETTER); //1023上传提示函
		mapToOperation.put("operatorAccount", userDetailOfVenture.getAccount()); // 操作账号先写的是死值
		mapToOperation.put("operatorRole", Role.VENTURE_MANAGEMENT_POS); //操作角色必定是【风管岗】
		mapToOperation.put("operatorPerson", userDetailOfVenture.getPerson_name()); // 操作人先写死
		mapToOperation.put("operatorTime", currentTime); // 操作时间为当前操作时间
		mapToOperation.put("hash", "hash");//=============================================================>>待定TODO
		promptLetterDao.insertOperationHistory(mapToOperation); // 将提示函泛函添加上传的操作记录信息插入到操作记录表中
	}
}
