package com.fuzamei.web;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.fastjson.JSON;
import com.fuzamei.constant.HintMSG;
import com.fuzamei.constant.Role;
import com.fuzamei.constant.StatusId;
import com.fuzamei.entity.ContractRecord;
import com.fuzamei.entity.ContractStatusTracking;
import com.fuzamei.entity.UserDetail;
import com.fuzamei.service.AttachmentService;
import com.fuzamei.service.ContractManagementService;
import com.fuzamei.service.UserAuthoricationService;
import com.fuzamei.utils.HashXiZhiUtil;
import com.fuzamei.utils.JSONUtil;
import com.fuzamei.utils.PageDTO;
import com.fuzamei.utils.RelativePathUtil;
import com.fuzamei.utils.ValidationUtil;

/**
 * 合同管理模块
 * @author ylx
 *
 */
@Controller
@RequestMapping("/contract_management")
public class ContractManagementAction {
	
	@Autowired
	private ContractManagementService contractManagementService;
	
	@Autowired
	private UserAuthoricationService userAuthoricationService;
	
	@Autowired
	private AttachmentService attachmentService;
	
	private static final int ROW_NUM = 10; // 分页每页显示数据的数量
	/**
	 * 
	* @Title: queryContractStatusTracking
	* @Description: TODO(查询合同状态跟踪表的信息，总表的信息)
	* @param @param data
			businessId:之前业务申报模块下来的id号
			contractParty:签约方，也是在之前业务申报模块中获取的这个信息
			{
				"userId":"xxx",
				"page":"1",
				"businessId":"",
				"contractParty":"",
				"startTime":"",
				"endTime":""
			}
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2018年1月8日 上午11:30:09
	* @throws
	 */
	@RequestMapping(value="/queryContractStatusTracking",method=RequestMethod.POST)
	@ResponseBody
	private Map<String, Object> queryContractStatusTracking(@RequestBody String data){
		try {
			System.out.println("查询合同状态跟踪表....");
			@SuppressWarnings("unchecked")
			Map<String,Object> map = JSON.parseObject(data, Map.class);
			int userId = ValidationUtil.checkAndAssignInt(map.get("userId"));
			//只有管理员，省分行业务主管部门，省分行法律部才有权限可以查看
			userAuthoricationService.checkUserAuthority(userId, Role.ADMIN,Role.BUSINESS_CHIEF_DEP_OF_PBB,Role.LAW_DEP_OF_PBB);
			int page = ValidationUtil.checkMinAndAssignInt(map.get("page"),1);
			Integer businessId = ValidationUtil.checkAndAssignNullIntegerIfIsBlank(map.get("businessId"));
			String contractParty = ValidationUtil.checkBlankStringAndAssignEmptyIfIsBlank(map.get("contractParty"));
			long startTime = ValidationUtil.checkAndAssignDefaultLong(map.get("startTime"), 0L);
			long endTime = ValidationUtil.checkAndAssignDefaultLong(map.get("endTime"), Long.MAX_VALUE);
			Map<String, Object> mapToService = new LinkedHashMap<String, Object>();
			mapToService.put("startPage", (page - 1) * ROW_NUM);
			mapToService.put("businessId", businessId);
			mapToService.put("contractParty", contractParty);
			//如果时间顺序错位，要重新进行排位
			mapToService.put("startTime", startTime);
			if (startTime <= endTime) {
				mapToService.put("endTime", endTime);
			} else {
				mapToService.put("endTime", Long.MAX_VALUE);
			}
			mapToService.put("rowNum", ROW_NUM); // 默认每页显示数据是10条，可根据需求修改分页数量
			PageDTO pageDto=contractManagementService.queryContractStatusTracking(mapToService);
			return JSONUtil.getJsonMap(200, true, HintMSG.QUERY_SUCCESS, pageDto);
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, HintMSG.QUERY_FAIL+":"+e.getMessage(), null);
		}
	}
	
	/**
	 * 
	* @Title: queryContractRecordByBusinessId
	* @Description: TODO(在合同状态跟踪表的基础上查询其中一条记录中的)
	* @param @param data
	{
		"userId":"",
		"page":"1",
		"businessId":""
	}
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2018年1月8日 下午2:50:36
	* @throws
	 */
	@RequestMapping(value="/queryContractRecordByBusinessId",method=RequestMethod.POST)
	@ResponseBody
	private Map<String, Object> queryContractRecordByBusinessId(@RequestBody String data){
		try {
			System.out.println("查询合同状态跟踪表....");
			@SuppressWarnings("unchecked")
			Map<String,Object> map = JSON.parseObject(data, Map.class);
			int userId = ValidationUtil.checkAndAssignInt(map.get("userId"));
			//只有管理员，省分行业务主管部门，省分行法律部才有权限可以查看
			UserDetail userDetail = userAuthoricationService.checkUserAuthority(userId, Role.ADMIN,Role.BUSINESS_CHIEF_DEP_OF_PBB,Role.LAW_DEP_OF_PBB);
			int page = ValidationUtil.checkMinAndAssignInt(map.get("page"),1);
			//这里的businessId是必查条件，所以不能默认为null，应该强制转换成int类型
			Integer businessId = ValidationUtil.checkAndAssignInt(map.get("businessId"));
			Map<String, Object> mapToService = new LinkedHashMap<String, Object>();
			mapToService.put("startPage", (page - 1) * ROW_NUM);
			mapToService.put("businessId", businessId);
			mapToService.put("userDetail", userDetail);
			mapToService.put("rowNum", ROW_NUM); // 默认每页显示数据是10条，可根据需求修改分页数量
			PageDTO pageDto=contractManagementService.queryContractRecord(mapToService);
			return JSONUtil.getJsonMap(200, true, HintMSG.QUERY_SUCCESS, pageDto);
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, HintMSG.QUERY_FAIL+":"+e.getMessage(), null);
		}
	}
	
	/**
	 * 上传新版本合同操作
	* @Title: uploadNewVersionContract
	* @Description: TODO(这里用一句话描述这个方法的作用)
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2018年1月8日 下午7:40:51
	* @throws
	 */
	@RequestMapping(value="/uploadNewVersionContract")
	@ResponseBody
	private Map<String, Object> uploadNewVersionContract(@RequestParam("file") MultipartFile file,
														 @RequestParam("userId") Object uId,
														 @RequestParam("modifyOpinion") String mOpinion,
														 @RequestParam("businessId") Object bId,
														 HttpServletRequest request){
		try {
			System.out.println("上传新版本合同操作....");
			if(file.isEmpty()) throw new RuntimeException("文件不能为空");
			String filename = file.getOriginalFilename();
			if(!filename.endsWith(".pdf")||!filename.endsWith(".doc")||!filename.endsWith(".docx")) throw new RuntimeException("文件上传格式不正确");
			int userId = ValidationUtil.checkAndAssignInt(uId);
			//只有管理员，省分行业务主管部门，省分行法律部才有权限可以查看
			UserDetail userDetail = userAuthoricationService.checkUserAuthority(userId, Role.ADMIN,Role.BUSINESS_CHIEF_DEP_OF_PBB,Role.LAW_DEP_OF_PBB);
			//这里的businessId是必查条件，所以不能默认为null，应该强制转换成int类型
			Integer businessId = ValidationUtil.checkAndAssignInt(bId);
			String modifyOpinion = ValidationUtil.checkBlankAndAssignString(mOpinion);
			ContractStatusTracking contractStatusTracking = contractManagementService.getContractStatusTrackingByBusinessId(businessId);
			ContractRecord contractRecord = contractManagementService.getLatestContractRecordByBusinessId(businessId);
			if(contractStatusTracking==null || contractRecord==null) throw new RuntimeException("无该合同信息");
			//如果已经确认过报错
			if(contractStatusTracking.getStatus_id()==StatusId.CONFIRMED) throw new RuntimeException("合同已确认，不能提交");//已确认id是8
			
			//确定好上传的文件的顶级父级路径=======================================================================>>待定TODO
			String path = request.getServletContext().getRealPath("");			//SP是系统分隔符
			String relativePath = RelativePathUtil.formatPath("/"+userId, "");	//生成文件的相对保存路径
			
			//暂时生成一个临时文件
			String tempPath = path+"/temp~~~~~"+filename;
			File tempFile = new File(tempPath);
			file.transferTo(tempFile);
			
			String hash = HashXiZhiUtil.getMD5Checksum(tempPath);//文件hash的计算待定
			tempFile.delete();//将临时文件先删除
			
			int contractId = attachmentService.generateAtachmentId();//生成不重复的附件id号
			
			Map<String, Object> mapToService = new LinkedHashMap<String, Object>();
			mapToService.put("contractId", contractId);
			mapToService.put("contractName", filename);
			mapToService.put("modifyOpinion", modifyOpinion);
			mapToService.put("userDetail", userDetail);
			mapToService.put("contractStatusTracking", contractStatusTracking);
			mapToService.put("contractRecord", contractRecord);
			mapToService.put("url", relativePath+filename);
			mapToService.put("hash", hash);	//文件hash待定
			contractManagementService.uploadContractByUser(mapToService);
			
			String pathFile=path+relativePath+filename;
			File newFile = new File(pathFile);
			//==========文件上传开始============
			File dir=new File(path+relativePath);
			if(!dir.exists()){
				dir.mkdirs();
			}
			file.transferTo(newFile);											//将---合同---文件上传到指定的文件夹下
			//==========文件上传结束============
			
			return JSONUtil.getJsonMap(200, true, HintMSG.UPLOAD_SUCCESS, null);
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, HintMSG.UPLOAD_FAIL+":"+e.getMessage(), null);
		}
	}
	
	/**
	 * 
	* @Title: downloadContractFiles
	* @Description: TODO(下载合同文件)
	* @param @param request
	* @param @param response
	* @param @param url
	* @param @param uId
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2018年1月9日 上午11:43:14
	* @throws
	 */
	@RequestMapping(value="/downloadContractFiles")
	@ResponseBody
	private Map<String, Object> downloadContractFiles(HttpServletRequest request,HttpServletResponse response,
			@RequestParam("url") String url,
			@RequestParam("userId") Object uId) {
		try {
			int userId=ValidationUtil.checkAndAssignInt(uId);
//			userAuthoricationService.checkUserAuthority(userId,Role.ADMIN,Role.BUSINESS_CHIEF_DEP_OF_PBB,Role.LAW_DEP_OF_PBB);	//查看用户非空和权限
			userAuthoricationService.checkUserAuthority(userId);//(暂时将权限对所有人全部放开)
			url=ValidationUtil.checkBlankAndAssignString(url);							//对传过来的url进行非空校验
			String fileName = request.getServletContext().getRealPath("")+url; 		//SP是系统分隔符
			//如果文件不存在直接抛出异常
			if(!new File(fileName).exists()){
				throw new RuntimeException(HintMSG.FILE_NOT_FOUND);
			}
			InputStream bis = new BufferedInputStream(new FileInputStream(new File(fileName)));  
			String filename = url.substring(url.lastIndexOf("/")+1);  //截取文件名
			filename = URLEncoder.encode(filename,"UTF-8");
			response.addHeader("Content-Disposition", "attachment;filename=" + filename);    
			response.setContentType("multipart/form-data");   
			BufferedOutputStream out = new BufferedOutputStream(response.getOutputStream());
			int len = 0;  
			while((len = bis.read()) != -1){  
				out.write(len);  
				out.flush(); 
			}  
			out.close();
			bis.close();
			return JSONUtil.getJsonMap(200, true, HintMSG.DOWNLOAD_SUCCESS, null);
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, HintMSG.DOWNLOAD_FAIL+":"+e.getMessage(), null);
		}
	}
	
	/**
	 * 
	* @Title: confirmContractContent
	* @Description: TODO(对合同进行确认)
	* @param @param data
	* 前端传过来的json数据格式
	{
		"userId":"xxx",
		"businessId":"",
		"version":""
	}
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2018年1月9日 上午11:48:05
	* @throws
	 */
	@RequestMapping(value="/confirmContractContent")
	@ResponseBody
	private Map<String, Object> confirmContractContent(@RequestBody String data){
		try {
			System.out.println("对合同记录表进行确认....");
			@SuppressWarnings("unchecked")
			Map<String,Object> map = JSON.parseObject(data, Map.class);
			int userId = ValidationUtil.checkAndAssignInt(map.get("userId"));
			UserDetail userDetail = userAuthoricationService.checkUserAuthority(userId,Role.ADMIN,Role.BUSINESS_CHIEF_DEP_OF_PBB,Role.LAW_DEP_OF_PBB);	//查看用户非空和权限
			Integer businessId = ValidationUtil.checkAndAssignInt(map.get("businessId"));
			Integer version = ValidationUtil.checkAndAssignInt(map.get("version"));
			ContractStatusTracking contractStatusTracking = contractManagementService.getContractStatusTrackingByBusinessId(businessId);
			ContractRecord contractRecord = contractManagementService.getCurrentContractRecordByBusinessId(businessId,version);//根据业务id和版本号获取当前合同记录信息
			if(contractStatusTracking==null || contractRecord==null) throw new RuntimeException("无该合同信息");
			//如果已经确认过报错
			if(contractStatusTracking.getStatus_id()==StatusId.CONFIRMED) throw new RuntimeException("合同已确认，不能再确认");//已确认id是8
			
			Map<String, Object> mapToService = new LinkedHashMap<String, Object>();
			mapToService.put("contractRecord", contractRecord);	//文件hash待定
			mapToService.put("userDetail", userDetail);	//文件hash待定
			contractManagementService.updateContractByUser(mapToService);
			
			return JSONUtil.getJsonMap(200, true, HintMSG.OPERATION_SUCCESS, null);
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, HintMSG.OPERATION_FAIL+":"+e.getMessage(), null);
		}
	}
	
	
}
